package xic;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.StringTokenizer;
import java.util.TreeSet;

/**
 * Keeps track of statistics-related messages and timing information
 */
public class Statistics {
	private class StatsItem implements Comparable<StatsItem> {
		long startTime, totalTime;
		int orderId;
		String name;
		List<String> messages;
		
		public StatsItem(String name, int orderId) {
			this.name = name;
			this.orderId = orderId;
			this.messages = new ArrayList<String>();
		}
		
		public int compareTo(StatsItem o) {
			if (o.orderId < orderId)
				return 1;
			else if (o.orderId > orderId)
				return -1;
			else
				return 0;
		}
		
		public void appendTo(StringBuffer sb) {
			sb.append(" - Stage '"+name+"'");
			for (int i=14-name.length(); i>=0; --i)
				sb.append(' ');
			sb.append("(Total time: " +totalTime + " ms)\n");
			for (String msg : messages) {
				boolean first = true;
				StringTokenizer tokenizer = new StringTokenizer(msg, "\r\n");
				while (tokenizer.hasMoreTokens()) {
					if (first)
						sb.append("   * ");
					else
						sb.append("     ");
					sb.append(tokenizer.nextToken()).append('\n');
					first = false;
				}
			}
		}
	};
	
	private HashMap<String, StatsItem> stats;
	private int stageCounter;

	public Statistics() {
		stats = new HashMap<String, StatsItem>();
		stageCounter = 0;
	}

	public void startTimer(String stageName) {
		if (!stats.containsKey(stageName))
			stats.put(stageName, new StatsItem(stageName, stageCounter++));
		
		StatsItem item = stats.get(stageName);
		item.startTime = System.currentTimeMillis();
	}

	public void stopTimer(String stageName) {
		StatsItem item = stats.get(stageName);
		item.totalTime += System.currentTimeMillis() - item.startTime;
	}

	public void log(String stageName, String msg) {
		if (!stats.containsKey(stageName))
			stats.put(stageName, new StatsItem(stageName, stageCounter++));
		stats.get(stageName).messages.add(msg);
	}

	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append("=================== Statistics ===================\n");
		for (StatsItem item : new TreeSet<StatsItem>(stats.values()))
			item.appendTo(buf);
		buf.append("==================================================");
		return buf.toString();
	}
}
