package xic.analysis;

import java.util.Map;
import java.util.Set;

import xic.Statistics;
import xic.ir.IRCall;
import xic.ir.IRMem;
import xic.ir.IRMove;
import xic.ir.IRTemp;


public class DeadCodeElimination {
	public static void transform(Statistics stats, int pass, Map<String, CFGNode> cfgs) {
		int deleted = 0;
		for (String name : cfgs.keySet()) {
			deleted += transform(name, cfgs.get(name));
			if (deleted > 0)
				stats.log("dce", "Pass " + pass + ": '" + name + "': removed "
						+ deleted + " statements");
		}
	}

	private static int transform(String funcName, CFGNode node) {
		int deleted = 0;
		Set<CFGNode> nodes = node.flatten();
		EssentialVariables ess = new EssentialVariables(nodes);
		ess.solve();

		for (CFGNode n : nodes) {
			if (!(n.value() instanceof IRMove))
				continue;
			IRMove move = (IRMove) n.value();
			if (move.target() instanceof IRMem)
				continue;
			String tempName = ((IRTemp) move.target()).name();
			if (move.expr() instanceof IRCall)
				continue;

			/* Only keep MOVEs, which write to a variable that is essential right after the CFG node */
			Set<String> essential = ess.getEssentialVariablesAfter(n);
			if (!essential.contains(tempName)) {
				n.removeSelf();
				deleted++;
			}
		}
		return deleted;
	}
}
