package xic.analysis;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import xic.Statistics;
import xic.analysis.Substitutable.SubstitutableFlowInfo;
import xic.ir.IRExpr;
import xic.ir.IRStmt;
import xic.ir.IRTemp;
import xic.ir.StructHashSet.StructWrapper;
import xic.visitors.RewritingVisitor;


// Implementation of Expression Propagation
public class LiveRangeSubstitution {
	public static void transform(Statistics stats, int pass, Map<String, CFGNode> cfgs, int max_live) {
		Map<StructWrapper, IRExpr> replaceMap = new HashMap<StructWrapper, IRExpr>();
		RewritingVisitor visitor = new RewritingVisitor(replaceMap, false);
		
		for (String funcName : cfgs.keySet()) {
			CFGNode headNode = cfgs.get(funcName);
			Set<CFGNode> nodeSet = headNode.flatten();
			
			LiveVariables live = new LiveVariables(nodeSet);
			live.solve();
			PropagatedExpressions prop = new PropagatedExpressions(nodeSet);
			prop.solve();
			
			Substitutable subst = new Substitutable(headNode.flatten(), headNode, prop, live, max_live);
			subst.solve();
			int numReplacements = 0;

			// replace a temp x in IRStmt with y if (x,y) is in the pair list
			for (CFGNode node : nodeSet) {
				if (node.value() instanceof IRStmt) {
					IRStmt stmt = (IRStmt) node.value();
					SubstitutableFlowInfo info = (SubstitutableFlowInfo) subst.getFlowInfo(node);

					replaceMap.clear();
					for (String temp : subst.in(node)) 
						//if(info.uniqueExprs.containsKey(temp))
							replaceMap.put(new StructWrapper(new IRTemp(temp)), 
									(IRExpr)info.uniqueExprs.get(temp));
					
					visitor.setKnownExpressions(replaceMap);
					
					node.setValue(visitor.visit(stmt));
					numReplacements += visitor.getNumRewrites();
				}
			}
			if (numReplacements > 0)
				stats.log("subst", "Pass " + pass + ": '" + funcName
						+ "': substituted " + numReplacements + " expressions");
		}
	}
}
