package xic.analysis;

import java.util.HashSet;
import java.util.Set;

import xic.InternalCompilerException;
import xic.codegen.AsmDirective;
import xic.codegen.Instr;
import xic.codegen.LabelInstr;
import xic.ir.IRNode;


/**
 * Live variables analysis - only implemented for assembly for now.
 */
public class LiveVariables extends DataFlowAnalysis {
	/// Value class used during live variable analysis
	public static class LVValue implements Value {
		/* List of variable names */
		protected Set<String> vars;

		public LVValue() {
			vars = new HashSet<String>();
		}

		public LVValue(Set<String> vars) {
			this.vars = new HashSet<String>(vars);
		}

		public Value copy() {
			return new LVValue(vars);
		}

		public void meet(Value v) {
			vars.addAll(((LVValue) v).vars);
		}
		
		public String toString() {
			return vars.toString();
		}

		public int hashCode() {
			return vars.hashCode();
		}
		
		public Set<String> vars() {
			return vars;
		}
		
		public boolean equals(Object obj) {
			if (!(obj instanceof LVValue))
				return false;
			return vars.equals(((LVValue) obj).vars);
		}

		public void setNullMeet() {
			vars.clear();
		}
	};

	/// Dataflow-related information associated with each node
	public static class LVFlowInfo extends FlowInfo {
		public Set<String> def, use;
		
		public LVFlowInfo(Object o) {
			use = new HashSet<String>();
			def = new HashSet<String>();
			entry = new LVValue();
			exit = new LVValue();

			/* Instr values */
			if (o instanceof Instr)
				((Instr) o).analyze(use, def);
			else if (o instanceof AsmDirective)
				; // Do nothing
			else if (o instanceof LabelInstr)
				; // Do nothing

			/* IRNode values */
			else if (o instanceof IRNode) {
				IRNode node = (IRNode) o;
				node.findWrittenTemps(def);
				node.findReadTemps(use);	
			} else if (o instanceof String)
				; // Do nothing
			else 
				throw new InternalCompilerException("Analysis not yet implemented for " + o +"("+o.getClass()+")");
		}
	};
	
	public LiveVariables(Set<CFGNode> nodes) {
		super(nodes);
		
		dir = Direction.BACKWARD;
		top = new LVValue();
	}
	
	protected FlowInfo createFlowInfo(CFGNode node) {
		return new LVFlowInfo(node.value());
	}
	
	public Set<String> getLiveVariablesBefore(CFGNode node) {
		return ((LVValue) ((LVFlowInfo) getFlowInfo(node)).exit).vars;
	}

	public Set<String> getLiveVariablesAfter(CFGNode node) {
		return ((LVValue) ((LVFlowInfo) getFlowInfo(node)).entry).vars;
	}

	public void F(FlowInfo flowInfo) {
		LVFlowInfo f = (LVFlowInfo) flowInfo;
		Set<String> exit = ((LVValue) f.exit).vars;
		Set<String> entry = ((LVValue) f.entry).vars;
		
		exit.clear();
		exit.addAll(entry);
		exit.removeAll(f.def);
		exit.addAll(f.use);
	}
}
