package xic.ast;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.xi.Position;
import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

/**
 * Groups a series of statements into a block
 */
public class Block extends Stmt {
	private List<Stmt> stmts;
	private Map<String, TypeNode> symbolTable;

	public Block() {
		super(null);
		stmts = new ArrayList<Stmt>();
	}

	public Block(Position pos) {
		super(pos);
		stmts = new ArrayList<Stmt>();
	}

	public Block(Position pos, List<Stmt> stmts, Map<String, TypeNode> symbolTable) {
		super(pos);
		this.stmts = stmts;
		this.symbolTable = symbolTable;
	}

	/**
	 * Create a new block with a single statement
	 */
	public Block(Stmt stmt) {
		this();
		appendStmt(stmt);
	}

	/**
	 * Copy constructor, makes a shallow copy while
	 * providing new position information.
	 */
	public Block(Position pos, Block block) {
		super(pos);
		this.stmts = block.stmts();
	}

	public Map<String, TypeNode> getSymbolTable() {
		return symbolTable;
	}
	
	public Block setSymbolTable(Map<String, TypeNode> symbolTable) {
		if (symbolTable == this.symbolTable)
			return this;
		Block b = (Block) copy();
		b.symbolTable = symbolTable;
		return b;
	}
	
	public void appendStmt(Stmt stmt) {
		stmts.add(stmt);
		extendPositionTo(stmt.position());
	}

    public ASTNode visitChildren(NodeVisitor v) {
    	boolean modified = false;

    	ArrayList<Stmt> results = new ArrayList<Stmt>();
    	for (Stmt stmt : stmts) {
    		Stmt newStmt = (Stmt) v.visit(this, stmt);
    		if (newStmt != stmt)
    			modified = true;
    		results.add(newStmt);
    	}

		if (modified)
			return new Block(position, results, symbolTable);

		return this;
	}
    
    public Copyable copy() {
		return new Block(position, new ArrayList<Stmt>(stmts), symbolTable);
    }

	public Iterable<VisualizableTreeNode> children() {
		ArrayList<VisualizableTreeNode> result 
			= new ArrayList<VisualizableTreeNode>(stmts.size());
		result.addAll(stmts);
		return result;
	}

	public List<Stmt> stmts() {
		return stmts;
	}

	public String label() {
		return "block";
	}
}
