package xic.ast;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.xi.Position;
import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

/**
 * Global function declaration
 */
public class FuncDecl extends Decl implements Typed {
	private String name;
	private TupleDecl param;
	private TypeNode retType;
	private Stmt body;
	private TypeNode type;
	private Map<String, TypeNode> symbolTable;

	public FuncDecl(Position pos, String name, TupleDecl param,
			TypeNode retType, Stmt body) {
		this(pos, name, param, retType, body, null);
	}

	public FuncDecl(Position pos, String name, TupleDecl param,
			TypeNode retType, Stmt body, Map<String, TypeNode> symbolTable) {
		super(pos);
		this.name = name;
		this.param = param;
		this.retType = retType;
		this.body = body;
		this.symbolTable = symbolTable;
	}

	public String name() {
		return name;
	}

	public TupleDecl param() {
		return param;
	}

	public boolean isImplemented() {
		return body != null;
	}

	public boolean isProcedure() {
		return retType == null;
	}

	public TypeNode retType() {
		return retType;
	}

	public Stmt body() {
		return body;
	}

	public TypeNode getType() {
		return type;
	}

	public Typed setType(TypeNode type) {
		if (type == this.type)
			return this;

		FuncDecl e = (FuncDecl) copy();
		e.type = type;
		return e;
	}

	public Map<String, TypeNode> getSymbolTable() {
		return symbolTable;
	}
	
	public FuncDecl setSymbolTable(Map<String, TypeNode> symbolTable) {
		if (symbolTable == this.symbolTable)
			return this;
		FuncDecl f = (FuncDecl) copy();
		f.symbolTable = symbolTable;
		return f;
	}

	public Iterable<VisualizableTreeNode> children() {
		List<VisualizableTreeNode> result = new ArrayList<VisualizableTreeNode>();
		result.add(param);

		if (!isProcedure())
			result.add(retType);
		
		if (isImplemented())
			result.add(body);
		return result;
	}

    public ASTNode visitChildren(NodeVisitor v) {
    	TupleDecl param = (TupleDecl) v.visit(this, this.param);
    	TypeNode retType = (TypeNode) v.visit(this, this.retType);
    	Stmt body = (Stmt) v.visit(this, this.body);

		if (param != this.param || retType != this.retType || body != this.body)
			return new FuncDecl(position, this.name, param, retType, body, symbolTable);

		return this;
	}

	public String label() {
		return "funcDecl " + name;
	}

	public List<String> getNames() {
		return Arrays.asList(new String[] { name });
	}

	public Copyable copy() {
		return new FuncDecl(position, this.name, param, retType, body, symbolTable);
	}
}
