package xic.ast;

import java.util.ArrayList;
import java.util.List;

import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.xi.Position;
import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

/**
 * Represents an if statement with an optional else block
 */
public class If extends Stmt {
	private Expr guard;
	private Stmt consequent, alternative;

	public If(Position pos, Expr guard, Stmt consequent, Stmt alternative) {
		super(pos);
		this.guard = guard;
		this.consequent = consequent;
		this.alternative = alternative;
	}
	
	public Iterable<VisualizableTreeNode> children() {
		List<VisualizableTreeNode> result = new ArrayList<VisualizableTreeNode>();
		
		result.add(guard);
		result.add(consequent);
		
		if (hasAlternative())
			result.add(alternative);
		return result;
	}

	public boolean hasAlternative() {
		return alternative != null;
	}

	public Expr guard() {
		return guard;
	}

	public Stmt consequent() {
		return consequent;
	}

	public Stmt alternative() {
		return alternative;
	}

    public ASTNode visitChildren(NodeVisitor v) {
		Expr guard = (Expr) v.visit(this, this.guard);
		Stmt consequent = (Stmt) v.visit(this, this.consequent);
		Stmt alternative = (Stmt) v.visit(this, this.alternative);

		if (guard != this.guard || consequent != this.consequent 
				|| alternative != this.alternative)
			return new If(position, guard, consequent, alternative); 

		return this;
	}
    
    public Copyable copy() {
		return new If(position, guard, consequent, alternative); 
    }

	public String label() {
		return "if";
	}
}