package xic.ast;

import java.util.ArrayList;
import java.util.List;

import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.xi.Position;
import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

/**
 * Represents a return statement in the AST. The
 * return expression is optional.
 */
public class Return extends Stmt {
	private Expr expr;

	public Return(Position pos) {
		super(pos);
	}
		
	public Return(Position pos, Expr expr) {
		super(pos);
		if (expr instanceof TupleExpr && ((TupleExpr) expr).exprCount() == 0)
			expr = null;
		this.expr = expr;
	}

	/**
	 * Copy constructor, makes a shallow copy while
	 * providing new position information.
	 */
	public Return(Position pos, Return ret) {
		super(pos);
		this.expr = ret.expr();
	}

	public Expr expr() {
		return expr;
	}
	
	public boolean hasExpr() {
		return expr != null;
	}

	public Iterable<VisualizableTreeNode> children() {
		List<VisualizableTreeNode> result = new ArrayList<VisualizableTreeNode>();

		if (hasExpr())
			result.add(expr);
		return result;
	}
	
    public ASTNode visitChildren(NodeVisitor v) {
    	Expr expr = (Expr) v.visit(this, this.expr);

		if (expr != this.expr)
			return new Return(position, expr);

		return this;
	}

	public String label() {
		return "return";
	}

	public Copyable copy() {
		return new Return (position, expr);
	}
	
	
}
