package xic.ast;

import java.util.ArrayList;
import java.util.List;

import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.xi.Position;
import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

/**
 * Stores expressions grouped as a tuple
 */
public class TupleExpr extends Expr {
	private List<Expr> subExprs;

	public TupleExpr() {
		super(null);
		this.subExprs = new ArrayList<Expr>();
	}

	public TupleExpr(Position pos) {
		super(pos);
		this.subExprs = new ArrayList<Expr>();
	}

	public TupleExpr(Position pos, List<Expr> subExprs) {
		super(pos);
		this.subExprs = subExprs;
	}

	public TupleExpr(Expr expr) {
		this();
		appendExpr(expr);
	}

	/**
	 * Copy constructor, makes a shallow copy while
	 * providing new position information.
	 */
	public TupleExpr(Position pos, TupleExpr te) {
		super(pos);
		this.subExprs = new ArrayList<Expr>(te.subExprs());
	}

	public void appendExpr(Expr expr) {
		subExprs.add(expr);
		extendPositionTo(expr.position());
	}

	public List<Expr> subExprs() {
		return subExprs;
	}

	public Iterable<VisualizableTreeNode> children() {
		ArrayList<VisualizableTreeNode> result 
			= new ArrayList<VisualizableTreeNode>(subExprs.size());
		result.addAll(subExprs);
		return result;
	}

	public String label() {
		return "tupleExpr";
	}

    public ASTNode visitChildren(NodeVisitor v) {
    	boolean modified = false;

    	ArrayList<Expr> results = new ArrayList<Expr>();
    	for (Expr expr : subExprs) {
    		Expr newExpr = (Expr) v.visit(this, expr);
    		if (newExpr != expr)
    			modified = true;
    		results.add(newExpr);
    	}

		if (modified)
			return new TupleExpr(position, results);

		return this;
	}
    
    public Expr peel() {
    	return subExprs.get(0);
    }

    public Expr remainder() {
		TupleExpr te = new TupleExpr();
		TupleType type = new TupleType();
		DataSource ds = new DataSource();
		for (int i=1; i<subExprs.size(); ++i) {
			type.appendType(subExprs.get(i).getType());
			te.appendExpr(subExprs.get(i));
			ds.add(subExprs.get(i).getDataSource());
		}
		return (Expr) te.setDataSource(ds).setType(type);
    }

	public Copyable copy() {
		Expr expr = new TupleExpr (position, new ArrayList<Expr> (subExprs));
    	expr.type = type;
    	expr.dataSource = dataSource;
    	return expr;
	}

	public int exprCount() {
		return subExprs.size();
	}	
}