package xic.ast;

import xic.ErrorSink;
import xic.InternalCompilerException;
import xic.visitors.NodeVisitor;
import edu.cornell.cs.cs4120.xi.CompilationException;
import edu.cornell.cs.cs4120.xi.Position;

/**
 * References a type by name -- this will later be resolved to
 * a concrete type, such as a <code>ClassType</code>
 */
public class TypeIdentifier extends TypeNode {
	private String name;

	public TypeIdentifier(Position pos, String name) {
		super(pos);
		this.name = name;
	}

	public String name() {
		return name;
	}
	
	public String label() {
		return "typeId " + name;
	}
	
	public String toString() {
		return name;
	}
	
	public ASTNode visitChildren(NodeVisitor v) {
		return this;
	}

	public Copyable copy() {
    	return new Identifier(position, name);
	}

	public boolean isIdenticalType(TypeNode node) {
	      
		throw new InternalCompilerException("isIdenticalType() called on unresolved type:" + name);
	}

	public boolean isSubtypeOf(TypeNode node) {
		throw new InternalCompilerException("isSubtypeOf() called on unresolved type: " + name);
	}
	
	public int size() {
		return 1;
	}
	
	public void match (TypeNode node, ErrorSink sink) {
		if (node instanceof TypeIdentifier) {
			TypeIdentifier ti = (TypeIdentifier) node;
			if ( !this.name.equals(ti.name))
				sink.addError(new CompilationException("Unmatched name in type identifier, excepted: "
						+this.name+", got: "+ti.name, node.position()));;
		}
		else
			sink.addError(new CompilationException("Unmatched type, excepted: "
					+this.label()+", got: "+node.label(), node.position()));
	}
}
