package xic.ast;

import java.util.Arrays;

import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.xi.Position;
import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

/**
 * Represents a unary expression. The particular 
 * operation type is denoted by an element of 
 * the <code>Type</code> enumeration.
 */
public class UnaryExpr extends Expr {
	public enum OpType {
		MINUS,
		NOT
	};

	private Expr expr;
	private OpType opType;

	public UnaryExpr(Position pos, OpType type, Expr expr) {
		super(pos);
		this.expr = expr;
		this.opType = type;
	}
	
	public Expr expr() {
		return expr;
	}
	
	public OpType opType() {
		return opType;
	}

	public Iterable<VisualizableTreeNode> children() {
		return Arrays.asList(new VisualizableTreeNode[] {expr});
	}

	public String label() {
		switch (opType) {
			case MINUS: return "-";
			case NOT: return "!";
			default: return "invalid";
		}
	}

	public ASTNode visitChildren(NodeVisitor v) {
		Expr expr = (Expr) v.visit(this, this.expr);
		
		if(expr != this.expr)
			return new UnaryExpr(position, opType, expr);
		return this;
	}

	public Copyable copy() {
		Expr expr = new UnaryExpr (position, opType, this.expr);
    	expr.type = type;
    	expr.dataSource = dataSource;
    	return expr;
	}
}
