package xic.ast;

import java.util.ArrayList;
import java.util.List;

import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.xi.Position;
import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

/**
 * Describes a list of use statements. These have
 * to appear at the beginning of a compilation unit.
 */
public class UseBlock extends ASTNode {
	private List<UseStmt> stmts;

	public UseBlock() {
		super(null);
		stmts = new ArrayList<UseStmt>();
	}
	
	public UseBlock(Position pos, List<UseStmt> stmts) {
		super(pos);
		this.stmts = stmts;
	}

	public UseBlock(UseStmt stmt) {
		super(stmt.position());
		stmts = new ArrayList<UseStmt>();
		stmts.add(stmt);
	}

	public void appendUseStmt(UseStmt stmt) {
		stmts.add(stmt);
		extendPositionTo(stmt.position());
	}
	
	public List<UseStmt> stmts() {
		return stmts;
	}

	public Iterable<VisualizableTreeNode> children() {
		ArrayList<VisualizableTreeNode> result = new ArrayList<VisualizableTreeNode>(
				stmts.size());
		result.addAll(stmts);
		return result;
	}

	public String label() {
		return "useBlock";
	}
	
    public Copyable copy() {
		return new UseBlock(position, new ArrayList<UseStmt>(stmts)); 
    }

	public ASTNode visitChildren(NodeVisitor v) {
		boolean modified = false;

    	ArrayList<UseStmt> results = new ArrayList<UseStmt>();
    	
		for (UseStmt stmt : stmts) {
    		UseStmt newStmt = (UseStmt) v.visit(this, stmt);
    		if (newStmt != stmt)
    			modified = true;
    		results.add(newStmt);
    	}

		if (modified)
			return new UseBlock(position, results);

		return this;
	}
}
