package xic.codegen.matchers;

import xic.Configuration;
import xic.Configuration.Target;
import xic.codegen.AbstractTile;
import xic.codegen.AsmDirective;
import xic.codegen.InstrList;
import xic.codegen.Matcher;
import xic.codegen.Tile;
import xic.codegen.Tiler;
import xic.ir.IRCtor;
import xic.ir.IRNode;
import xic.ir.NameGenerator;

public class CtorMatcher implements Matcher {
	protected static class CtorTile extends AbstractTile {
		private String name;

		public CtorTile(String name) {
			this.name = name;
		}

		public int localCost() {
			return 0;
		}

		public String translate(InstrList il, NameGenerator ng) {
			boolean osx = Configuration.TARGET == Target.DARWIN;

			if (osx) {
				il.add(new AsmDirective(".mod_init_func"));
			} else {
				il.add(new AsmDirective(".section .ctors"));
			}
			il.add(new AsmDirective("\t.align "+ (osx ? Configuration.LOG_WORD_SIZE : Configuration.WORD_SIZE)));
			if (Configuration.WORD_SIZE == 4) {
				il.add(new AsmDirective("\t.long " + name));
			} else {
				il.add(new AsmDirective("\t.quad " + name));
			}
			il.add(new AsmDirective("\t.text"));
			il.add(new AsmDirective(""));

			return null;
		}
	};

	public Tile match(IRNode node, Tiler tiler) {
		if (node instanceof IRCtor) {
			IRCtor data = (IRCtor) node;
			return new CtorTile(data.name());
		}
		
		return null;
	}
}
