package xic.ir;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import xic.ast.ASTNode;
import xic.ast.Copyable;
import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

public class IRDataSeq extends IRStmt {
	private String name;
	private List<IRExpr> values;
	private boolean readOnly;

	public IRDataSeq(String name, boolean readOnly, List<IRExpr> values) {
		this.name = name;
		this.values = values;
		this.readOnly = readOnly;
	}

	public IRDataSeq(String name, boolean readOnly) {
		this.name = name;
		this.values = new ArrayList<IRExpr>();
		this.readOnly = readOnly;
	}

	public void appendConst(IRConst constValue) {
		values.add(constValue);
	}

	public void appendName(IRName nameValue) {
		values.add(nameValue);
	}

	public List<IRExpr> values() {
		return values;
	}

	public String name() {
		return name;
	}

	public ASTNode visitChildren(NodeVisitor v) {
		boolean modified = false;

		ArrayList<IRExpr> results = new ArrayList<IRExpr>();
		for (IRExpr val : values) {
			IRExpr newVal = (IRExpr) v.visit(this, val);
			if (newVal instanceof IRESeq)
				newVal = ((IRESeq) newVal).expr();
			if (newVal != val)
				modified = true;
			results.add(newVal);
		}

		if (modified)
			return new IRDataSeq(name, readOnly, results);

		return this;
	}

	public Copyable copy() {
		return new IRDataSeq(name, readOnly, new ArrayList<IRExpr>(values));
	}

	public Copyable deepCopy() {
		List<IRExpr> exprCopy = new ArrayList<IRExpr>();
		for (IRExpr v : values)
			exprCopy.add((IRExpr) v.deepCopy());
		return new IRDataSeq(name, readOnly, exprCopy);
	}

	public boolean containsCalls() {
		return false;
	}

	public boolean containsMemReads() {
		return false;
	}

	public boolean containsNonParamMemReads() {
		return false;
	}

	public boolean containsMemWrites() {
		return false;
	}

	public void findReadTemps(Set<String> tempNames) {
	}

	public void findWrittenTemps(Set<String> tempNames) {
	}

	public int computeMaximumCallArguments() {
		return 0;
	}

	public int computeMaximumCallResults() {
		return 0;		
	}

	public int nodeCount() {
		int nodeCount = 0;
		for (IRNode n : values)
			nodeCount += n.nodeCount();
		return nodeCount + 1;
	}

	public Iterable<VisualizableTreeNode> children() {
		ArrayList<VisualizableTreeNode> result = new ArrayList<VisualizableTreeNode>(
				values.size());
		result.addAll(values);
		return result;
	}

	public String label() {
		if (readOnly)
			return "DSEQ_RO";
		else
			return "DSEQ";
	}

	public boolean equalsTree(Object object) {
		if (!(object instanceof IRSeq))
			return false;
		IRDataSeq other = (IRDataSeq) object;
		if (other.values.size() != values.size())
			return false;

		for (int i = 0; i < values.size(); ++i)
			if (!values.get(i).equalsTree(other.values.get(i)))
				return false;
		return true;
	}

	public int treeHashCode() {
		int code = 17;
		for (IRExpr c : values)
			code = 37 * code + c.treeHashCode();
		return code;
	}

	public boolean isReadOnly() {
		return readOnly;
	}
}
