package xic.ir;

import java.util.Arrays;
import java.util.Set;

import xic.ast.ASTNode;
import xic.ast.Copyable;
import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

public class IRJump extends IRStmt {
	private IRExpr target;
	
	public IRJump(IRExpr expr) {
		this.target = expr;
	}

	public IRExpr target() {
		return target;
	}

	public Iterable<VisualizableTreeNode> children() {
		return Arrays.asList(new VisualizableTreeNode[] {
			target
		});
	}

    public ASTNode visitChildren(NodeVisitor v) {
    	IRExpr expr = (IRExpr) v.visit(this, this.target);

		if (expr != this.target)
			return new IRJump(expr);

		return this;
	}

	public String label() {
		return "JUMP";
	}
	
	public boolean containsCalls() {
		return target.containsCalls();
	}

	public boolean containsMemReads() {
		return target.containsMemReads();
	}
	
	public boolean containsMemWrites() {
		return target.containsMemWrites();
	}

	public int computeMaximumCallArguments() {
		return 0;
	}

	public int computeMaximumCallResults() {
		return 0;		
	}

	public void findReadTemps(Set<String> tempNames) {
		target.findReadTemps(tempNames);
	}

	public void findWrittenTemps(Set<String> tempNames) {
		target.findWrittenTemps(tempNames);
	}
	
	public int nodeCount() {
		return 1 + target.nodeCount();
	}

	public Copyable copy() {
		return new IRJump(target);
	}
	
	public Copyable deepCopy() {
		return new IRJump((IRExpr) target.deepCopy());
	}
	
	public boolean equalsTree(Object object) {
		if (!(object instanceof IRJump))
			return false;
		IRJump other = (IRJump) object;
		return other.target.equalsTree(target);
	}
	
	public int treeHashCode() {
		return target.treeHashCode();
	}
};