package xic.ir;

import java.io.IOException;
import java.io.OutputStream;
import java.io.StringWriter;
import java.util.Iterator;

import polyglot.util.CodeWriter;
import polyglot.util.OptimalCodeWriter;
import edu.cornell.cs.cs4120.util.TreePrinter;
import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

public class IRPrinter implements TreePrinter {
	protected CodeWriter writer;

	public IRPrinter(CodeWriter writer) {
		this.writer = writer;
	}

	public static String IRToString(IRNode node) {
		return IRToString(node, 80);
	}

	public static String IRToString(IRNode node, int maxWidth) {
		StringWriter writer = new StringWriter();
		IRPrinter printer = new IRPrinter(
				new OptimalCodeWriter(writer, maxWidth));
		printer.print(node);
		String result = writer.getBuffer().toString();
		// remove trailing newline
		return result.substring(0, result.length()-1);
	}

	public IRPrinter(OutputStream o) {
		this.writer = new OptimalCodeWriter(o, 80);
	}

	public void print(VisualizableTreeNode node) {
		try {
			printHelper(node, 0);
			writer.newline();
			writer.flush();
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	private void printHelper(VisualizableTreeNode node, int depth) throws IOException {
		writer.write(node.label());
		Iterator<VisualizableTreeNode> it = node.children().iterator();
		boolean newLine = node instanceof IRSeq || node instanceof IRESeq || node instanceof IRCompUnit || node instanceof IRFuncDecl;
		boolean hasChildren = it.hasNext();
	
		if (hasChildren)
			writer.write("(");

		if (node instanceof IRDataSeq) {
			IRDataSeq ds = (IRDataSeq) node;
			writer.write("\"" + ds.name() + "\", ");
		}

		if (newLine)
			writer.begin(2-node.label().length());

		if (hasChildren && newLine)
			writer.newline();

		if (hasChildren) {
			for (; it.hasNext();) {
				VisualizableTreeNode child = it.next();
				printHelper(child, depth+1);
				if (it.hasNext()) {
					writer.write(", ");
					if (node instanceof IRSeq || node instanceof IRESeq || node instanceof IRCompUnit) {
						writer.newline();
					} else {
						/* Try harder not to break during the first few depths */
						int level = 1;
						if (depth < 6)
							level = 2;
						writer.allowBreak(2, level, "", 0);
					}
				}
			}
		}
		if (newLine) {
			writer.end();
			writer.newline();
		}
		if (node instanceof IRCJump) {
			IRCJump jump = (IRCJump) node;
			String trueLabel = jump.hasTrueLabel() ? jump.trueLabel() : "<none>";
			String falseLabel = jump.hasFalseLabel() ? jump.falseLabel() : "<none>";
			writer.write(", " + trueLabel);
			writer.write(", " + falseLabel);
		}
		if (hasChildren) {
			writer.write(")");
		}
	}
}
