package xic.visitors;

import java.util.Map;

import xic.ast.ASTNode;
import xic.ir.IRExpr;
import xic.ir.IRMem;
import xic.ir.IRMove;
import xic.ir.StructHashSet.StructWrapper;


/**
 * Replaces certain expressions by temporaries. Used by copy propagation
 * and partial redundancy elimination.
 */
public class RewritingVisitor extends NodeVisitor {
	private Map<StructWrapper, IRExpr> knownExpressions;
	private int numRewrites = 0;
	private boolean recurse;

	public RewritingVisitor(Map<StructWrapper, IRExpr> knownExpressions, boolean recurse) {
		this.knownExpressions = knownExpressions;
		this.recurse = recurse;
	}

	public RewritingVisitor(Map<StructWrapper, IRExpr> knownExpressions) {
		this(knownExpressions, true);
	}

	public Map<StructWrapper, IRExpr> getKnownExpressions() {
		return knownExpressions;
	}
	
	public void setKnownExpressions(Map<StructWrapper, IRExpr> knownExpressions) {
		this.knownExpressions = knownExpressions;
		this.numRewrites = 0;
	}
	
	public int getNumRewrites() {
		return numRewrites;
	}

	protected ASTNode leave(ASTNode parent, ASTNode nOld, ASTNode nNew,
			NodeVisitor vNew) {
		if (nOld instanceof IRExpr) {
			StructWrapper wrapper = new StructWrapper((IRExpr) nOld);
			if (knownExpressions.containsKey(wrapper)) {
				numRewrites++;
				if (recurse)
					return visit((IRExpr) knownExpressions.get(wrapper).deepCopy());
				else
					return (IRExpr) knownExpressions.get(wrapper).deepCopy();
			}
		} else if (nOld instanceof IRMove
				&& ((IRMove) nOld).target() instanceof IRMem
				&& !(((IRMove) nNew).target() instanceof IRMem)) {
			/* Allow to replace expressions inside a  MOVE(MEM(...),..),
			   but do not allow to replace the MEM itself -- this is
			   needed because of PRE */
			IRMove oldMove = (IRMove) nOld;
			IRMove newMove = (IRMove) nNew;
			IRMem oldMoveMem = (IRMem) oldMove.target();
			return new IRMove(
					new IRMem((IRExpr) visit(oldMoveMem.expr())),
					newMove.expr()
			);
		} else if (nOld instanceof IRMove
				&& !(((IRMove) nOld).target() instanceof IRMem)
				&& (((IRMove) nOld).target() != ((IRMove) nNew).target())) {
			/* Do not allow to rewrite the left-hand side of a move */
			IRMove oldMove = (IRMove) nOld;
			IRMove newMove = (IRMove) nNew;
			numRewrites--;
			return new IRMove((IRExpr) oldMove.target().deepCopy(), newMove.expr());
		}
		return nNew;
	}
}
