package xic;


import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.util.Iterator;

import org.junit.Test;

import xic.ast.ASTNode;
import xic.ast.ArrayType;
import xic.ast.Assignment;
import xic.ast.Block;
import xic.ast.CompUnit;
import xic.ast.DeclBlock;
import xic.ast.ErrorReporter;
import xic.ast.ErrorStmt;
import xic.ast.FuncDecl;
import xic.ast.Identifier;
import xic.ast.IntLiteral;
import xic.ast.PrimitiveType;
import xic.ast.Stmt;
import xic.ast.TupleDecl;
import xic.ast.UseBlock;
import xic.parser.IotaParser;
import edu.cornell.cs.cs4120.util.VisualizableTreeNode;
import edu.cornell.cs.cs4120.xi.CompilationException;

public class ParserErrorTest {
	// use statements must come before global stmts
	@Test(expected = CompilationException.class)
	public void negativeTest1() {
		IotaParser.getAST(
			    "x:int = 2;\n" +
			    "use io;"
		);
	}

	//Umatched brace
	@Test(expected = CompilationException.class)
	public void negativeTest2() {
		IotaParser.getAST(
				"p(): int[] = {\n" +
				   "if (a==b) {\n" +
						"b=0;\n" +
					"}\n" +
					"x = a b c;"
		);
	}

	// If w/o guard -- recovered error test
	@Test
	public void negativeTest3() {
		ASTNode parsedAST = IotaParser.getAST(
				"p(): int[] = {\n" +
					"a = 3;\n" +
				  	"if " +
					"b = 3;\n" +
				"}\n" 
		);

		assertTrue(hasErrors(parsedAST));

		PositionImpl pos = new PositionImpl("test4",0,0,0,0); // unused
		Assignment assign1 = new Assignment(pos, new Identifier(pos, "a"), new IntLiteral(pos, 3));
		Stmt error = new ErrorStmt(pos);

		Block block = new Block(pos);

		block.appendStmt(assign1);
		block.appendStmt(error);
		
		FuncDecl func = new FuncDecl(pos, "p", new TupleDecl(), 
				new ArrayType(pos, new PrimitiveType(pos, PrimitiveType.Type.INTEGER),null),block);

		UseBlock useBlock = new UseBlock();
		DeclBlock gloBlock = new DeclBlock();
		gloBlock.appendDecl(func);
		CompUnit program = new CompUnit(pos, useBlock, gloBlock);
	
		ErrorReporter.checkNodes(program, parsedAST, false);
	}

	//Dangling else
	@Test
	public void negativeTest4() {
		ASTNode parsedAST= IotaParser.getAST(
				"p(): int[] = {\n" +
				   "if (a==b) {\n" +
						"b=0;\n" +
					"}\n" +
					"else if (a<b)\n" +
					"a=1;\n" +
					"else\n" +
					"b=1;\n" +
					"else\n" +
					"a=2;\n" +
				"}\n"
		);
		assertTrue(hasErrors(parsedAST));
	}

	//Unmatched nested if expr
	@Test
	public void negativeTest5() {
		ASTNode parsedAST= IotaParser.getAST(
				"p(): int[] = {\n" +
				   "if (a==b) {\n" +
						"b=0;\n" +
					"}\n" +
					"else if (((a<b))))\n" +
					"a=1;" +
				"}\n"
		);
		assertTrue(hasErrors(parsedAST));
	}

	//Invalid top-level 
	@Test(expected = CompilationException.class)
	public void negativeTest6() {
		IotaParser.getAST(
				"use io;\n" +
				"if (false)\n" +
					"b:int=0;\n" +
				"else if (true)\n" +
					"x:int=1;\n" +
				"\n"
		);
	}

	//Invalid use
	@Test(expected = CompilationException.class)
	public void negativeTest7() {
		IotaParser.getAST(
				"use \"io\";\n"
		);
	}

	//Invalid function call
	@Test
	public void negativeTest8() {
		ASTNode parsedAST= IotaParser.getAST(
				"p(): int[] = {\n" +
				   "p _;\n" +
				"}\n");
		assertTrue(hasErrors(parsedAST));
	}

	//Invalid array access
	@Test
	public void negativeTest9() {
		ASTNode parsedAST= IotaParser.getAST(
				"p(): int[] = {\n" +
				   "(p 1)(2);\n" +
				"}\n");
		assertTrue(hasErrors(parsedAST));
	}

	//Invalid underscore
	@Test
	public void negativeTest10() {
		ASTNode parsedAST= IotaParser.getAST(
				"p(): int[] = {\n" +
				   "_ = _;\n" +
				"}\n");
		assertTrue(hasErrors(parsedAST));
	}

	//Naked length expression in function call
	@Test
	public void negativeTest11() {
		ASTNode parsedAST= IotaParser.getAST(
				"p(): int[] = {\n" +
				   "x = a length b a;\n" +
				"}\n");
		assertTrue(hasErrors(parsedAST));
	}

	//Invalid tuple declaration
/*	No longer a parser error: handled by typechecker
 * @Test
	public void negativeTest12() {
		ASTNode parsedAST= IotaParser.getAST(
				"p(): int[] = {\n" +
				   "() = ();\n" +
				"}\n");
		assertTrue(hasErrors(parsedAST));
	}
*/
	// Using a statement as an initializer
	@Test(expected = CompilationException.class)
	public void negativeTest13() {
		IotaParser.getAST("x:int = break;");
	}

	// It is forbidden to declare variables of the void type
	// Using a statement as an initializer
	@Test(expected = CompilationException.class)
	public void negativeTest14() {
		IotaParser.getAST("x:() = 1;");
	}

	// The set of permissible lvalues does not include literals
/*	No longer a parser error: handled by type checker
 * @Test
	public void negativeTest15() {
		ASTNode parsedAST = IotaParser.getAST("a() = {\n"
				+ "(a, (b, 3)) = (1);\n"
				+ "}");
		assertTrue(hasErrors(parsedAST));
	}
*/

	// The set of permissible lvalues does not include array lookups within tuples
	@Test
	public void negativeTest16() {
		ASTNode parsedAST = IotaParser.getAST("a() = {\n"
				+ "(b 2, x) = (0, 1);\n"
				+ "}");
		assertFalse(hasErrors(parsedAST));
	}

	// Assignments are statements
	@Test
	public void negativeTest17() {
		ASTNode parsedAST = IotaParser.getAST("a() = {\n"
			+ "  b=c=d;\n"
			+ "}");
		assertTrue(hasErrors(parsedAST));
	}

	private boolean hasErrors(ASTNode node) {
		Iterator<VisualizableTreeNode> children = node.children().iterator();
		while (children.hasNext()) {
			ASTNode item = (ASTNode) children.next();

			if (item instanceof ErrorStmt) {
				return true;
			}
			if (hasErrors(item))
				return true;
		}
		return false;
	}

	// Can't have a function call on an arbitrary expression
	@Test
	public void negativeTest18() {
		ASTNode parsedAST= IotaParser.getAST(
				"p(): int[] = {\n" +
				   "(a+1) b;\n" +
				"}\n");
		assertTrue(hasErrors(parsedAST));
	}
	
	@Test
	public void negativeTest19() {
		// Assignment statements of type:
		// "String"(2)
		// (1, 2, 3) 2
		ASTNode parsedAST= IotaParser.getAST(
				 "arraysAsStrings(s:int[]):int = {"
				+"    x:int = (1,2,3) 2;"
				+"    return x + \"Sic transit gloria mundi\"(2);"
				+"}"
				);
		assertTrue(hasErrors(parsedAST));
	}
	
	// Can't have initializations of attributes
	@Test(expected = CompilationException.class)
	public void negativeTest20() {
		ASTNode parsedAST= IotaParser.getAST(
				"class A { i: int = 3; }");
		assertTrue(hasErrors(parsedAST));
	}
}
