package xic;


import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import org.junit.Test;

import xic.ast.ASTNode;
import xic.ast.ArrayType;
import xic.ast.Assignment;
import xic.ast.BinaryExpr;
import xic.ast.Block;
import xic.ast.BoolLiteral;
import xic.ast.Call;
import xic.ast.ClassDecl;
import xic.ast.CompUnit;
import xic.ast.DeclBlock;
import xic.ast.DotExpr;
import xic.ast.ErrorReporter;
import xic.ast.FuncDecl;
import xic.ast.Identifier;
import xic.ast.If;
import xic.ast.IntLiteral;
import xic.ast.Length;
import xic.ast.LocalVarDecl;
import xic.ast.PrimitiveType;
import xic.ast.Return;
import xic.ast.StringLiteral;
import xic.ast.This;
import xic.ast.TupleDecl;
import xic.ast.TupleExpr;
import xic.ast.TupleType;
import xic.ast.TypeIdentifier;
import xic.ast.UnaryExpr;
import xic.ast.UnitType;
import xic.ast.UseBlock;
import xic.ast.UseStmt;
import xic.ast.VarDecl;
import xic.ast.While;
import xic.parser.IotaParser;
import edu.cornell.cs.cs4120.xi.Position;

public class ParserTest {
	/**
	 * Parses the GCD example and verifies it for correctness. The 
	 * AST tree below was manually annotated with position information,
	 * and this is included in the comparison.
	 */
	@Test()
	public void test1() {
		String unit = "<string>";
		ASTNode parsedAST = IotaParser.getAST(
			  "use io;\n"
			+ "gcd(a: int, b: int): int = {\n" 
			+ "  while (a != 0) {\n"
			+ "    if (a<b) b = b-a;\n"
			+ "    else a = a - b;\n" 
			+ "  }\n"
			+ "  return b;\n" 
			+ "}"
		);

		assertNotNull(parsedAST);

		Position posCompUnit = new PositionImpl(unit, 0, 7, 0, 0);
		Position posUse = new PositionImpl(unit, 0, 0, 0, 6);
		Position posParamAType  = new PositionImpl(unit, 1, 1, 7, 9);
		Position posParamBType = new PositionImpl(unit, 1, 1, 15, 17);
		Position posParamA  = new PositionImpl(unit, 1, 1, 4, 9);
		Position posParamB  = new PositionImpl(unit, 1, 1, 12, 17);
		Position posParams  = new PositionImpl(unit, 1, 1, 3, 18);
		Position posRetType  = new PositionImpl(unit, 1, 1, 21, 23);
		Position posFunc = new PositionImpl(unit, 1, 7, 0, 0);
		Position posFuncBlock = new PositionImpl(unit, 1, 7, 27, 0);
		Position posWhile = new PositionImpl(unit, 2, 5, 2, 2);
		Position posWhileBlock = new PositionImpl(unit, 2, 5, 17, 2);
		Position posWhileComparison = new PositionImpl(unit, 2, 2, 9, 14);
		Position posWhileComparisonA = new PositionImpl(unit, 2, 2, 9, 9);
		Position posWhileComparison0 = new PositionImpl(unit, 2, 2, 14, 14);
		Position posIfStmt = new PositionImpl(unit, 3, 4, 4, 18);
		Position posIfComparison = new PositionImpl(unit, 3, 3, 8, 10);
		Position posIfComparisonA = new PositionImpl(unit, 3, 3, 8, 8);
		Position posIfComparisonB = new PositionImpl(unit, 3, 3, 10, 10);
		Position posAssignment1 = new PositionImpl(unit, 3, 3, 13, 20);
		Position posAssignment1L = new PositionImpl(unit, 3, 3, 13, 13);
		Position posAssignment1R = new PositionImpl(unit, 3, 3, 17, 19);
		Position posAssignment1RB = new PositionImpl(unit, 3, 3, 17, 17);
		Position posAssignment1RA = new PositionImpl(unit, 3, 3, 19, 19);
		Position posAssignment2 = new PositionImpl(unit, 4, 4, 9, 18);
		Position posAssignment2L = new PositionImpl(unit, 4, 4, 9, 9);
		Position posAssignment2R = new PositionImpl(unit, 4, 4, 13, 17);
		Position posAssignment2RA = new PositionImpl(unit, 4, 4, 13, 13);
		Position posAssignment2RB = new PositionImpl(unit, 4, 4, 17, 17);
		Position posReturnStmt = new PositionImpl(unit, 6, 6, 2, 10);
		Position posReturnB = new PositionImpl(unit, 6, 6, 9, 9);

		Assignment gcdIfConseq = new Assignment(posAssignment1, new Identifier(
				posAssignment1L, "b"), new BinaryExpr(posAssignment1R,
				BinaryExpr.OpType.MINUS, new Identifier(posAssignment1RB, "b"),
				new Identifier(posAssignment1RA, "a")));

		Assignment gcdIfAlt = new Assignment(posAssignment2, new Identifier(
				posAssignment2L, "a"), new BinaryExpr(posAssignment2R,
				BinaryExpr.OpType.MINUS, new Identifier(posAssignment2RA, "a"),
				new Identifier(posAssignment2RB, "b")));

		If gcdIf = new If(posIfStmt, new BinaryExpr(posIfComparison,
				BinaryExpr.OpType.LT, new Identifier(posIfComparisonA, "a"),
				new Identifier(posIfComparisonB, "b")), new Block(gcdIfConseq), new Block(gcdIfAlt));
		
		Block gcdWhileBody = new Block();
		gcdWhileBody.appendStmt(gcdIf);
		gcdWhileBody = new Block(posWhileBlock, gcdWhileBody);

		While gcdWhileLoop = new While(posWhile, new BinaryExpr(posWhileComparison,
				BinaryExpr.OpType.NOT_EQUAL, new Identifier(posWhileComparisonA, "a"),
				new IntLiteral(posWhileComparison0, 0)), gcdWhileBody);
		
		Return gcdReturnStmt = new Return(posReturnStmt, new Identifier(posReturnB, "b"));

		Block gcdBody = new Block();
		gcdBody.appendStmt(gcdWhileLoop);
		gcdBody.appendStmt(gcdReturnStmt);
		gcdBody = new Block(posFuncBlock, gcdBody);

		/* Create the function signature and append it to the program */
		PrimitiveType gcdRetType = new PrimitiveType(posRetType, PrimitiveType.Type.INTEGER);
		TupleDecl gcdParams = new TupleDecl();
		gcdParams.appendTupleDecl(new TupleDecl(posParamA, "a", new PrimitiveType(posParamAType,
				PrimitiveType.Type.INTEGER)));
		gcdParams.appendTupleDecl(new TupleDecl(posParamB, "b", new PrimitiveType(posParamBType,
				PrimitiveType.Type.INTEGER)));
		gcdParams = new TupleDecl(posParams, gcdParams);

		FuncDecl gcdFunc = new FuncDecl(posFunc, "gcd", gcdParams, gcdRetType, gcdBody);
		UseBlock useBlock = new UseBlock(new UseStmt(posUse, "io"));
		DeclBlock globalStmtBlock = new DeclBlock();

		CompUnit program = new CompUnit(posCompUnit, useBlock, globalStmtBlock);
		globalStmtBlock.appendDecl(gcdFunc);

		ErrorReporter.checkNodes(program, parsedAST, true);
	}

	@Test()
	// for assignment example in spec
	public void test2() {
		ASTNode parsedAST = IotaParser.getAST(
			    "use io;\n"
			+   "a() = {"
			+   "  x:int = 2;" 
			+ 	"  y:(bool, int)=(true, 1);"
			+ 	"  z:int;"
			+ 	"  (b:bool, i:int) = y;" 
			+ 	"  s:int [ ] = \"Hello\";"
			+	"  (x:int, _, z:int) = (1,2,3);"
			+	"  a: int [] = (72, 101, 108, 108, 111);"
			+	"  a: int [] = \"Hello\";"
			+   "}"
		);
		PositionImpl pos = new PositionImpl("test2",0,0,0,0); // unused
		LocalVarDecl decl1 = new LocalVarDecl( pos, new TupleDecl(pos, "x", 
				new PrimitiveType (pos, PrimitiveType.Type.INTEGER)), new IntLiteral(pos, 2));
		
		TupleType boolint = new TupleType(pos);
		boolint.appendType( new PrimitiveType (pos, PrimitiveType.Type.BOOLEAN));
		boolint.appendType(new PrimitiveType (pos, PrimitiveType.Type.INTEGER));
		TupleDecl decl2l = new TupleDecl(pos, "y", boolint);
		TupleExpr decl2r = new TupleExpr(pos);
		decl2r.appendExpr( new BoolLiteral(pos, true));
		decl2r.appendExpr( new IntLiteral(pos, 1));
		LocalVarDecl decl2 = new LocalVarDecl(pos, decl2l, decl2r);
		
		
		LocalVarDecl decl3 = new LocalVarDecl(pos, new TupleDecl(pos, "z", 
				new PrimitiveType (pos, PrimitiveType.Type.INTEGER)));
		
		TupleDecl decl4l = new TupleDecl();
		decl4l.appendTupleDecl( new TupleDecl(pos, "b", new PrimitiveType(pos, PrimitiveType.Type.BOOLEAN)));
		decl4l.appendTupleDecl( new TupleDecl(pos, "i", new PrimitiveType(pos, PrimitiveType.Type.INTEGER)));
		LocalVarDecl decl4 = new LocalVarDecl(pos, decl4l, new Identifier(pos, "y"));
		
		ArrayType atype = new ArrayType(pos, new PrimitiveType(pos, PrimitiveType.Type.INTEGER),null);

		TupleDecl decl5l = new TupleDecl(pos, "s", atype);
		LocalVarDecl decl5 = new LocalVarDecl(pos, decl5l, new StringLiteral(pos, "Hello"));
		
		TupleDecl decl6l = new TupleDecl();
		decl6l.appendTupleDecl( new TupleDecl(pos, "x", new PrimitiveType (pos, PrimitiveType.Type.INTEGER)));
		decl6l.appendTupleDecl( new TupleDecl(pos, (String) null, new UnitType(pos)));
		decl6l.appendTupleDecl( new TupleDecl(pos, "z", new PrimitiveType (pos, PrimitiveType.Type.INTEGER)));
		TupleExpr decl6r = new TupleExpr(pos);
		decl6r.appendExpr( new IntLiteral(pos, 1));
		decl6r.appendExpr( new IntLiteral(pos, 2));
		decl6r.appendExpr( new IntLiteral(pos, 3));
		LocalVarDecl decl6 = new LocalVarDecl(pos, decl6l, decl6r);
		
		TupleDecl decl7l = new TupleDecl(pos, "a", 
				new ArrayType(pos, new PrimitiveType(pos, PrimitiveType.Type.INTEGER),null));
		
		TupleExpr decl7r = new TupleExpr(pos);
		decl7r.appendExpr( new IntLiteral(pos, 72));
		decl7r.appendExpr( new IntLiteral(pos, 101));
		decl7r.appendExpr( new IntLiteral(pos, 108));
		decl7r.appendExpr( new IntLiteral(pos, 108));
		decl7r.appendExpr( new IntLiteral(pos, 111));
		LocalVarDecl decl7 = new LocalVarDecl(pos, decl7l, decl7r);

		TupleDecl decl8l = new TupleDecl(pos, "a", 
				new ArrayType(pos, new PrimitiveType(pos, PrimitiveType.Type.INTEGER),null));

		LocalVarDecl decl8 = new LocalVarDecl(pos, decl8l, new StringLiteral(pos, "Hello"));
		
		UseBlock useBlock = new UseBlock( new UseStmt(pos, "io"));
		DeclBlock globalStmtBlock = new DeclBlock();
		Block block = new Block();
		globalStmtBlock.appendDecl(new FuncDecl(pos, "a", new TupleDecl(), null, block));

		block.appendStmt(decl1);
		block.appendStmt(decl2);
		block.appendStmt(decl3);
		block.appendStmt(decl4);
		block.appendStmt(decl5);
		block.appendStmt(decl6);
		block.appendStmt(decl7);
		block.appendStmt(decl8);
		CompUnit program = new CompUnit(pos, useBlock, globalStmtBlock);

		ErrorReporter.checkNodes(program, parsedAST, false);
	} 
	
	@Test()
	// for variables example in spec
	public void test3() {
		ASTNode parsedAST = IotaParser.getAST(
				  "assign() = {\n" 
				+	"x=x+1;\n" 
				+	"y = (false,0);\n"
				+	"b = ! b;\n"
				+	"s = (1,2,3,4);\n" 
				+	"a n = n;"
				+ "}"
		);
		
		PositionImpl pos = new PositionImpl("test3",0,0,0,0); // unused
		Assignment assign1 = new Assignment(pos, new Identifier(pos, "x"), new BinaryExpr(pos, 
				BinaryExpr.OpType.PLUS, new Identifier(pos, "x"), new IntLiteral(pos, 1)));
		
		TupleExpr assign2r = new TupleExpr(pos);
		assign2r.appendExpr( new BoolLiteral(pos, false));
		assign2r.appendExpr( new IntLiteral(pos, 0));
		Assignment assign2 = new Assignment(pos, new Identifier(pos, "y"), assign2r);
		
		Assignment assign3 = new Assignment(pos, new Identifier(pos, "b"), 
				new UnaryExpr(pos, UnaryExpr.OpType.NOT, new Identifier(pos, "b")));
		
		TupleExpr assign4r = new TupleExpr(pos);
		assign4r.appendExpr( new IntLiteral(pos, 1));
		assign4r.appendExpr( new IntLiteral(pos, 2));
		assign4r.appendExpr( new IntLiteral(pos, 3));		
		assign4r.appendExpr( new IntLiteral(pos, 4));		
		Assignment assign4 = new Assignment(pos, new Identifier(pos, "s"), assign4r);
		
		Call call = new Call(pos, new Identifier(pos, "a"), 
				new TupleExpr(new Identifier(pos, "n")));
		
		Assignment assign5 = new Assignment(pos, call, new Identifier(pos, "n"));
				
		Block block = new Block(pos);
		block.appendStmt(assign1);
		block.appendStmt(assign2);
		block.appendStmt(assign3);
		block.appendStmt(assign4);
		block.appendStmt(assign5);
		
		FuncDecl gcdFunc = new FuncDecl(pos, "assign", new TupleDecl(), null, block);
		UseBlock useBlock = new UseBlock();
		DeclBlock gloBlock = new DeclBlock();
		gloBlock.appendDecl(gcdFunc);
		CompUnit program = new CompUnit(pos, useBlock, gloBlock);
		assertEquals(IotaParser.astToString(program),IotaParser.astToString(parsedAST));
		ErrorReporter.checkNodes(program, parsedAST, false);
	}
	
	@Test()
	// to test the precedence of operators and function calls
	public void test4() {
		ASTNode parsedAST = IotaParser.getAST(
				  "p( ): int[] = {\n" 
				+ 	"b = !false | b | c& !d\n!= true;"
				+	"x = -a%2+b-c*-3/-d >= 0;"
				+	"x = a b c;"
				+ "}"
		);
		
		PositionImpl pos = new PositionImpl("test4",0,0,0,0); // unused

		UnaryExpr notFalse = new UnaryExpr(pos, UnaryExpr.OpType.NOT, 
				new BoolLiteral(pos, false));
		BinaryExpr left = new BinaryExpr (pos, BinaryExpr.OpType.OR,
				notFalse, new Identifier(pos, "b"));
		
		UnaryExpr notD = new UnaryExpr(pos, UnaryExpr.OpType.NOT, 
				new Identifier(pos, "d"));
		BinaryExpr expr2 = new BinaryExpr(pos, BinaryExpr.OpType.NOT_EQUAL, notD, new BoolLiteral(pos, true));
		
		BinaryExpr right = new BinaryExpr(pos, BinaryExpr.OpType.AND,
				new Identifier(pos, "c"), expr2);
		BinaryExpr righthand = new BinaryExpr(pos, BinaryExpr.OpType.OR, left, right);

		Assignment assign1 = new Assignment(pos, new Identifier(pos, "b"), righthand);
		
		UnaryExpr negA = new UnaryExpr(pos, UnaryExpr.OpType.MINUS, 
				new Identifier(pos, "a"));
		left = new BinaryExpr(pos, BinaryExpr.OpType.MODULO, negA, new IntLiteral(pos, 2));
		BinaryExpr left2 = new BinaryExpr(pos, BinaryExpr.OpType.PLUS, left, 
				new Identifier(pos, "b"));
		right = new BinaryExpr(pos, BinaryExpr.OpType.TIMES, new Identifier(pos, "c"), 
				new UnaryExpr(pos, UnaryExpr.OpType.MINUS, new IntLiteral(pos, 3)));
		UnaryExpr negD =  new UnaryExpr(pos, UnaryExpr.OpType.MINUS, new Identifier(pos, "d"));
		BinaryExpr right2 = new BinaryExpr(pos, BinaryExpr.OpType.DIVIDE, right, negD);
		BinaryExpr right3 = new BinaryExpr(pos, BinaryExpr.OpType.MINUS, left2, right2);
		BinaryExpr value = new BinaryExpr(pos, BinaryExpr.OpType.GEQ, right3, new IntLiteral(pos, 0));
		Assignment assign2 = new Assignment(pos, new Identifier(pos, "x"), value);
		
		TupleExpr tupleinCall = new TupleExpr( new Identifier(pos, "b"));
		Call leftCall = new Call(pos, new Identifier(pos, "a"), tupleinCall);
		Call nestedCall = new Call(pos, leftCall, new TupleExpr( new Identifier(pos, "c")));
		Assignment assign3 = new Assignment(pos, new Identifier(pos, "x"), nestedCall);
		
		Block block = new Block(pos);

		block.appendStmt(assign1);
		block.appendStmt(assign2);
		block.appendStmt(assign3);
		
		FuncDecl func = new FuncDecl(pos, "p", new TupleDecl(), 
				new ArrayType(pos, new PrimitiveType(pos, PrimitiveType.Type.INTEGER),null), block);

		UseBlock useBlock = new UseBlock();
		DeclBlock gloBlock = new DeclBlock();
		gloBlock.appendDecl(func);
		CompUnit program = new CompUnit(pos, useBlock, gloBlock);
		
		assertEquals(IotaParser.astToString(program), IotaParser.astToString(parsedAST));
	}
	
	@Test()
	// to test the dangling if-then-else, and the if statement
	public void test5() {
		ASTNode parsedAST = IotaParser.getAST(
				  "p( ): int = {\n" 
				+ "	if (!!!b) \n"
				+ "   if (c) \n"
				+ "     x=2; \n"
				+ "   else"
				+ "     x=3; \n"
				+ "	if (b) \n"
				+ "   x=1;\n"
				+ " if (c) \n"
				+ "   x=2; \n"
				+ " else"
				+ "   x=3; \n"
				+ "}"
		);
		
		PositionImpl pos = new PositionImpl("test5",0,0,0,0); // unused
		
		UnaryExpr u1 = new UnaryExpr(pos, UnaryExpr.OpType.NOT, new Identifier(pos, "b"));
		UnaryExpr u2 = new UnaryExpr(pos, UnaryExpr.OpType.NOT, u1);
		UnaryExpr u3 = new UnaryExpr(pos, UnaryExpr.OpType.NOT, u2);
		
		Assignment thenClause = new Assignment(pos, new Identifier(pos, "x"),	new IntLiteral(pos, 2));
		Assignment elseClause = new Assignment(pos, new Identifier(pos, "x"),	new IntLiteral(pos, 3));
		
		If nestedIf = new If(pos, new Identifier(pos, "c"), new Block(thenClause), new Block(elseClause));
		If topIf = new If(pos, u3, new Block(nestedIf), null);
		
		Assignment firstThen = new Assignment(pos, new Identifier(pos, "x"), new IntLiteral(pos, 1));
		If firstIf = new If(pos, new Identifier(pos, "b"), new Block(firstThen), null);
		
		Assignment secondThen = new Assignment(pos, new Identifier(pos, "x"), new IntLiteral(pos, 2));
		Assignment secondElse = new Assignment(pos, new Identifier(pos, "x"), new IntLiteral(pos, 3));
		If secondIf= new If(pos, new Identifier(pos, "c"), new Block(secondThen), new Block(secondElse));
				
		Block block = new Block(pos);
		block.appendStmt(topIf);
		block.appendStmt(firstIf);
		block.appendStmt(secondIf);
		
		FuncDecl func = new FuncDecl(pos, "p", new TupleDecl(), 
				new PrimitiveType (pos, PrimitiveType.Type.INTEGER), block);

		UseBlock useBlock = new UseBlock();
		DeclBlock gloBlock = new DeclBlock();
		gloBlock.appendDecl(func);
		CompUnit program = new CompUnit(pos, useBlock, gloBlock);
		
		ErrorReporter.checkNodes(program, parsedAST, false);
	}
	
	@Test()
	// to test function def with single statement
	public void test6() {
		ASTNode parsedAST = IotaParser.getAST(
				"min(x:int, y:int) = if (x<y) return x; else return y;"
		);
		
		PositionImpl pos = new PositionImpl("test6",0,0,0,0); // unused
		
		TupleDecl para = new TupleDecl();
		para.appendTupleDecl( new TupleDecl(pos, "x", new PrimitiveType(pos, PrimitiveType.Type.INTEGER)));
		para.appendTupleDecl( new TupleDecl(pos, "y", new PrimitiveType(pos, PrimitiveType.Type.INTEGER)));
		
		BinaryExpr less = new BinaryExpr(pos, BinaryExpr.OpType.LT, 
				new Identifier(pos, "x"), new Identifier(pos, "y"));
		If stmt = new If(pos, less, new Block(new Return(pos, new Identifier(pos, "x"))), 
				new Block(new Return(pos, new Identifier(pos, "y"))));
		
		FuncDecl func = new FuncDecl(pos, "min", para, null, new Block(stmt));

		UseBlock useBlock = new UseBlock();
		DeclBlock gloBlock = new DeclBlock();
		gloBlock.appendDecl(func);
		CompUnit program = new CompUnit(pos, useBlock, gloBlock);
		
		ErrorReporter.checkNodes(program, parsedAST, false);
	}
	
	@Test()
	//  to test nested type decl
	public void test7() {	
		ASTNode parsedAST = IotaParser.getAST(
				// doesn't make much sense, but is syntactically correct
			    "a() = { x:(int, bool, int [])[3][-2]; }"
		);

		PositionImpl pos = new PositionImpl("test7",0,0,0,0); // unused
		ArrayType intarray =  new ArrayType(pos, new PrimitiveType(pos, PrimitiveType.Type.INTEGER), null);
		TupleType tuple = new TupleType(pos);
		tuple.appendType( new PrimitiveType(pos, PrimitiveType.Type.INTEGER));
		tuple.appendType( new PrimitiveType(pos, PrimitiveType.Type.BOOLEAN));
		tuple.appendType( intarray);

		ArrayType nested1 = new ArrayType(pos, tuple, new UnaryExpr(pos,
				UnaryExpr.OpType.MINUS, new IntLiteral(pos, 2)));
		ArrayType nested2 = new ArrayType(pos, nested1, new IntLiteral(pos, 3));

		TupleDecl tupleDecl = new TupleDecl(pos, "x", nested2);
		LocalVarDecl decl =  new LocalVarDecl(pos, tupleDecl);
		
		UseBlock useBlock = new UseBlock();
		DeclBlock gloBlock = new DeclBlock();
		gloBlock.appendDecl(new FuncDecl(pos, "a", new TupleDecl(), null, new Block(decl)));
		CompUnit program = new CompUnit(pos, useBlock, gloBlock);

		ErrorReporter.checkNodes(program, parsedAST, false);
	}
	
	@Test()
	// a complete sort program
	public void test8() {
		ASTNode parsedAST = IotaParser.getAST(
			    "sort(a: int[]) = {"
			+	"  i:int = 0;"
			+	"  n:int = length a;"
			+	"  while (i < n) {"
			+	"    j:int = i;"
			+	"    while (j > 0) {"
			+	"      if (a(j-1) > a(j)) { // parens are optional on a(j)\n"
			+	"         swap:int = a(j);"
			+	"         a(j) = a(j-1);"
			+	"         a(j-1) = swap;"
			+	"	   }"
			+	"      j = j-1;"
			+	"    }"
			+	"    i = i+1;"
			+	"  }"
			+	"}"
		);

		PositionImpl pos = new PositionImpl("test8",0,0,0,0); // unused
		
		// if
		Block ifBlock = new Block(pos);
		
		Call aj = new Call(pos, new Identifier(pos, "a"), 
				new TupleExpr(new Identifier(pos, "j")));
		
		LocalVarDecl swapDecl = new LocalVarDecl(pos, 
				new TupleDecl(pos, "swap", new PrimitiveType(pos, PrimitiveType.Type.INTEGER)), aj);

		Call aj1 = new Call(pos, new Identifier(pos, "a"), 
				new TupleExpr(new BinaryExpr(pos, BinaryExpr.OpType.MINUS, new Identifier(pos, "j"), new IntLiteral(pos, 1))));
		
		Assignment assign1 = new Assignment(pos, aj, aj1);
		Assignment assign2 = new Assignment(pos, aj1, new Identifier(pos, "swap"));
		ifBlock.appendStmt( swapDecl);
		ifBlock.appendStmt( assign1);
		ifBlock.appendStmt( assign2);
		BinaryExpr ifGuard = new BinaryExpr(pos, BinaryExpr.OpType.GT, aj1, aj); // as long as we ignore the position, it's OK
		If ifStmt = new If(pos, ifGuard, ifBlock, null);
		
		// nested while
		Assignment assignI = new Assignment(pos, new Identifier(pos, "j"), 
			new BinaryExpr(pos, BinaryExpr.OpType.MINUS, new Identifier(pos, "j"), new IntLiteral(pos, 1)));
		Block nestedWhileBlock = new Block(pos);
		nestedWhileBlock.appendStmt(ifStmt);
		nestedWhileBlock.appendStmt(assignI);
		BinaryExpr whileCond = new BinaryExpr(pos, BinaryExpr.OpType.GT, 
				new Identifier(pos, "j"), new IntLiteral(pos, 0));
		While	nestedWhile = new While(pos, whileCond, nestedWhileBlock);
		
		// top while
		Block topWhileBlock = new Block(pos);
		LocalVarDecl jDecl = new LocalVarDecl(pos, 
				new TupleDecl(pos, "j", new PrimitiveType( pos, PrimitiveType.Type.INTEGER)), 
				new Identifier(pos, "i"));
		assignI = new Assignment(pos, new Identifier(pos, "i"),
				new BinaryExpr(pos,BinaryExpr.OpType.PLUS,new Identifier(pos, "i"),new IntLiteral(pos, 1)));
		topWhileBlock.appendStmt(jDecl);
		topWhileBlock.appendStmt(nestedWhile);
		topWhileBlock.appendStmt(assignI);
		While topWhile = new While(pos, 
				new BinaryExpr(pos, BinaryExpr.OpType.LT, new Identifier(pos, "i"), new Identifier(pos, "n")),
				topWhileBlock);
		
		// function body
		Block sortBody = new Block(pos);
		LocalVarDecl iDecl = new LocalVarDecl(pos, new TupleDecl(pos, "i", new PrimitiveType(pos, PrimitiveType.Type.INTEGER))
					, new IntLiteral(pos, 0));
		LocalVarDecl nDecl = new LocalVarDecl(pos, new TupleDecl(pos, "n", new PrimitiveType(pos, PrimitiveType.Type.INTEGER)) 
					, new Length(pos, new Identifier(pos, "a")));
		sortBody.appendStmt(iDecl);
		sortBody.appendStmt(nDecl);
		sortBody.appendStmt(topWhile);
		
		// function decl
		TupleDecl para = new TupleDecl(new TupleDecl(pos, "a",
				new ArrayType(pos, new PrimitiveType(pos,
						PrimitiveType.Type.INTEGER), null)));
		FuncDecl func = new FuncDecl(pos, "sort", para, null, sortBody);

		UseBlock useBlock = new UseBlock();
		DeclBlock gloBlock = new DeclBlock();
		gloBlock.appendDecl(func);
		CompUnit program = new CompUnit(pos, useBlock, gloBlock);
		assertEquals(IotaParser.astToString(program), IotaParser.astToString(parsedAST));
		ErrorReporter.checkNodes(program, parsedAST, false);
	}

	@Test()
	// assign to a nested tuple decl
	public void test9() {
		ASTNode parsedAST = IotaParser.getAST(
			    "a() = { (x:int, (y:bool, z:int)) = (1, (2,3));}"
		);
		PositionImpl pos = new PositionImpl("test9",0,0,0,0); // unused

		TupleDecl outer = new TupleDecl();
		outer.appendTupleDecl( new TupleDecl(pos, "x", new PrimitiveType(pos, PrimitiveType.Type.INTEGER)));		
		TupleDecl inner = new TupleDecl();
		inner.appendTupleDecl( new TupleDecl(pos, "y", new PrimitiveType(pos, PrimitiveType.Type.BOOLEAN)));
		inner.appendTupleDecl( new TupleDecl(pos, "z", new PrimitiveType(pos, PrimitiveType.Type.INTEGER)));
		outer.appendTupleDecl(inner);
		TupleExpr tup1 = new TupleExpr();
		tup1.appendExpr(new IntLiteral(pos, 1));
		TupleExpr tup2 = new TupleExpr();
		tup2.appendExpr(new IntLiteral(pos, 2));
		tup2.appendExpr(new IntLiteral(pos, 3));
		tup1.appendExpr(tup2);
		LocalVarDecl decl = new LocalVarDecl(pos, outer,tup1);

		UseBlock useBlock = new UseBlock();
		DeclBlock gloBlock = new DeclBlock();
		gloBlock.appendDecl(new FuncDecl(pos, "a", new TupleDecl(), null, new Block(decl)));
		CompUnit program = new CompUnit(pos, useBlock, gloBlock);
		
		ErrorReporter.checkNodes(program, parsedAST, false);
	}
	
	@Test()
	// a empty block and empty return test
	public void test10() {
		ASTNode parsedAST = IotaParser.getAST(
					"foo()={}\n"
				+	"emptyreturn () = {return;}");
		
		PositionImpl pos = new PositionImpl("test10",0,0,0,0); // unused
		
		Block block = new Block(pos);
		FuncDecl func = new FuncDecl(pos, "foo", new TupleDecl(), null, block);
		
		Block returnBlock = new Block(pos);
		returnBlock.appendStmt( new Return(pos));
		FuncDecl func1 = new FuncDecl(pos, "emptyreturn", new TupleDecl(), null, returnBlock);
		
		UseBlock useBlock = new UseBlock();
		DeclBlock gloBlock = new DeclBlock();
		gloBlock.appendDecl(func);
		gloBlock.appendDecl(func1);
		CompUnit program = new CompUnit(pos, useBlock, gloBlock);
		
		ErrorReporter.checkNodes(program, parsedAST, false);
	}

	@Test()
	// Chained function calls
	public void test11() {
		ASTNode parsedAST = IotaParser.getAST(
			  "func(a:int[])={x:int = a 2 (n == 1);}"
		);
		PositionImpl pos = new PositionImpl("test11",0,0,0,0); // unused
		TupleDecl para = new TupleDecl(new TupleDecl(pos, "a",
				new ArrayType(pos, new PrimitiveType(pos,
						PrimitiveType.Type.INTEGER), null)));
		
		Block funcBody = new Block(pos);

		TupleExpr nEQEQ1 = new TupleExpr();
		nEQEQ1.appendExpr(new BinaryExpr(pos, BinaryExpr.OpType.EQUAL, new Identifier(pos, "n"), new IntLiteral(pos, 1)));
		
		Call a2 = new Call(pos, new Identifier(pos, "a"), new TupleExpr(new IntLiteral(pos,2)));
		Call nestedCall = new Call(pos, a2, nEQEQ1);
		
		LocalVarDecl xDecl = new LocalVarDecl(pos, 
				new TupleDecl(pos, "x", new PrimitiveType(pos, PrimitiveType.Type.INTEGER)), 
				nestedCall);
		funcBody.appendStmt(xDecl);
		
		FuncDecl func = new FuncDecl(pos, "func", para, null, funcBody);
		UseBlock useBlock = new UseBlock();
		DeclBlock gloBlock = new DeclBlock();
		gloBlock.appendDecl(func);
		CompUnit program = new CompUnit(pos, useBlock, gloBlock);

		assertEquals(IotaParser.astToString(program), IotaParser.astToString(parsedAST));
		ErrorReporter.checkNodes(program, parsedAST, false);
	}

	@Test()
	// Test a class declaration
	public void testClassDecl() {
		ASTNode parsedAST = IotaParser.getAST(
				"class ColoredPoint extends Point {" +
				"	x, y: int;" +
				"   returnSelf() : ColoredPoint = {" +
				"      return this;" +
				"   }" +
				"}"
		);

		PositionImpl pos = new PositionImpl("test12",0,0,0,0); // unused

		Block block = new Block(pos);
		block.appendStmt(new Return(pos, new This(pos)));
		FuncDecl func = new FuncDecl(pos, "returnSelf", new TupleDecl(), 
				new TypeIdentifier(pos, "ColoredPoint"), block);
		TupleDecl td = new TupleDecl();
		td.appendTupleDecl(new TupleDecl(pos, "x", PrimitiveType.INTEGER_TYPE));
		td.appendTupleDecl(new TupleDecl(pos, "y", PrimitiveType.INTEGER_TYPE));
		VarDecl vd = new VarDecl(pos, PrimitiveType.INTEGER_TYPE, td, null, false);

		DeclBlock classBlock = new DeclBlock();
		classBlock.appendDecl(vd);
		classBlock.appendDecl(func);

		ClassDecl cd = new ClassDecl(pos, "ColoredPoint", "Point", classBlock, false);
		
		DeclBlock gloBlock = new DeclBlock();
		CompUnit program = new CompUnit(pos, new UseBlock(), gloBlock);
		gloBlock.appendDecl(cd);

		ErrorReporter.checkNodes(program, parsedAST, false);
	}
	@Test()
	// Test a dot expression
	public void testDotExpr() {
		ASTNode parsedAST = IotaParser.getAST(
				"class ColoredPoint extends Point {" +
				"	x, y: int;" +
				"   returnSelf() : ColoredPoint = {" +
				"      return this;" +
				"   }" +
				"   getX() : int = {" +
				"      return this.x;" +
				"   }" +
				"}"
		);
		PositionImpl pos = new PositionImpl("testDotExpr",0,0,0,0); // unused

		Block block = new Block(pos);
		block.appendStmt(new Return(pos, new This(pos)));
		
		Block block2 = new Block(pos);
		block2.appendStmt(new Return(pos, new DotExpr(pos, new This(pos), "x")));
		
		FuncDecl func = new FuncDecl(pos, "returnSelf", new TupleDecl(), 
				new TypeIdentifier(pos, "ColoredPoint"), block);
		FuncDecl func2 = new FuncDecl(pos, "getX", new TupleDecl(), 
				new PrimitiveType(pos,PrimitiveType.Type.INTEGER), block2);
		TupleDecl td = new TupleDecl();
		td.appendTupleDecl(new TupleDecl(pos, "x", PrimitiveType.INTEGER_TYPE));
		td.appendTupleDecl(new TupleDecl(pos, "y", PrimitiveType.INTEGER_TYPE));
		VarDecl vd = new VarDecl(pos,  PrimitiveType.INTEGER_TYPE, td, null, false);

		DeclBlock classBlock = new DeclBlock();
		classBlock.appendDecl(vd);
		classBlock.appendDecl(func);
		classBlock.appendDecl(func2);

		ClassDecl cd = new ClassDecl(pos, "ColoredPoint", "Point", classBlock, false);
		
		DeclBlock gloBlock = new DeclBlock();
		CompUnit program = new CompUnit(pos, new UseBlock(), gloBlock);
		gloBlock.appendDecl(cd);

		ErrorReporter.checkNodes(program, parsedAST, false);
	}
}
