package xic;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import org.junit.Test;

import xic.PositionImpl;
import xic.ast.ArrayType;
import xic.ast.PrimitiveType;
import xic.ast.TupleType;
import xic.ast.UnitType;


/**
 * This class performs a few rudimentary tests directly using the subtyping API.
 * More complex test cases can be found in <code>TypeCheckerTest</code> and
 * <code>TypeCheckerErrorTest</code>
 */
public class SubTypeTest {
	PositionImpl pos = new PositionImpl(null, 0, 0, 0, 0);

	@Test
	public void test1() {
		// Basic subtyping test for tuples
		TupleType type1 = new TupleType();
		type1.appendType(new PrimitiveType(pos, PrimitiveType.Type.INTEGER));
		type1.appendType(new PrimitiveType(pos, PrimitiveType.Type.BOOLEAN));
		type1.appendType(new PrimitiveType(pos, PrimitiveType.Type.INTEGER));

		TupleType type2 = new TupleType();
		type2.appendType(new PrimitiveType(pos, PrimitiveType.Type.INTEGER));
		type2.appendType(new PrimitiveType(pos, PrimitiveType.Type.BOOLEAN));
		type2.appendType(new PrimitiveType(pos, PrimitiveType.Type.INTEGER));

		assertTrue(type1.isSubtypeOf(type1));
		assertTrue(type1.isSubtypeOf(type2));
		assertTrue(type2.isSubtypeOf(type1));

		type2 = new TupleType();
		type2.appendType(new PrimitiveType(pos, PrimitiveType.Type.INTEGER));
		type2.appendType(new UnitType(pos));
		type2.appendType(new PrimitiveType(pos, PrimitiveType.Type.INTEGER));

		assertTrue(type1.isSubtypeOf(type2));
		assertFalse(type2.isSubtypeOf(type1));
	}

	@Test
	public void test2() {
		// Tuples <-> Arrays
		TupleType type1 = new TupleType();
		type1.appendType(new PrimitiveType(pos, PrimitiveType.Type.INTEGER));
		type1.appendType(new PrimitiveType(pos, PrimitiveType.Type.INTEGER));
		type1.appendType(new PrimitiveType(pos, PrimitiveType.Type.INTEGER));

		ArrayType type2 = new ArrayType(pos, new PrimitiveType(pos,
				PrimitiveType.Type.INTEGER), null);

		assertTrue(type1.isSubtypeOf(type2));
		assertFalse(type2.isSubtypeOf(type1));
	}

	@Test
	public void test3() {
		// Nesting
		TupleType type1 = new TupleType();
		type1.appendType(new PrimitiveType(pos, PrimitiveType.Type.INTEGER));
		type1.appendType(new PrimitiveType(pos, PrimitiveType.Type.BOOLEAN));
		type1.appendType(new PrimitiveType(pos, PrimitiveType.Type.INTEGER));

		TupleType type2 = new TupleType();
		type2.appendType(new PrimitiveType(pos, PrimitiveType.Type.INTEGER));
		type2.appendType(new UnitType(pos));
		type2.appendType(new PrimitiveType(pos, PrimitiveType.Type.INTEGER));

		assertTrue(type1.isSubtypeOf(type2));

		TupleType type3 = new TupleType();
		type3.appendType(type1);
		type3.appendType(new PrimitiveType(pos, PrimitiveType.Type.INTEGER));

		TupleType type4 = new TupleType();
		type4.appendType(type2);
		type4.appendType(new PrimitiveType(pos, PrimitiveType.Type.INTEGER));
		assertTrue(type3.isSubtypeOf(type4));
		assertFalse(type4.isSubtypeOf(type3));
	}
}
